@@ -67,6 +67,7 @@
 #include "visibletracks.h"
 #include "xml.h"
 #include "arrangercolumns.h"
+#include "tlist.h"
 
 namespace MusEGui {
 
@@ -88,21 +89,27 @@ ArrangerView::ArrangerView(QWidget* parent)
   scoreAllInOneMapper = new QSignalMapper(this);
 
   editSignalMapper = new QSignalMapper(this);
-  QShortcut* sc = new QShortcut(shortcuts[SHRT_DELETE].key, this);
-  sc->setContext(Qt::WindowShortcut);
-  connect(sc, SIGNAL(activated()), editSignalMapper, SLOT(map()));
-  editSignalMapper->setMapping(sc, CMD_DELETE);
 
   // Toolbars ---------------------------------------------------------
+
+  // NOTICE: Please ensure that any tool bar object names here match the names assigned 
+  //          to identical or similar toolbars in class MusE or other TopWin classes. 
+  //         This allows MusE::setCurrentMenuSharingTopwin() to do some magic
+  //          to retain the original toolbar layout. If it finds an existing
+  //          toolbar with the same object name, it /replaces/ it using insertToolBar(),
+  //          instead of /appending/ with addToolBar().
+
+  addToolBarBreak();
+  
   editTools = new EditToolBar(this, arrangerTools);
   addToolBar(editTools);
+  // Make sure name doesn't conflict with other TopWin edit toolbar object names.
   editTools->setObjectName("arrangerTools");
 
+  // Already has an object name.
   visTracks = new VisibleTracks(this);
   addToolBar(visTracks);
 
-
-
   connect(editTools, SIGNAL(toolChanged(int)), arranger, SLOT(setTool(int)));
   connect(visTracks, SIGNAL(visibilityChanged()), MusEGlobal::song, SLOT(update()) );
   connect(arranger, SIGNAL(editPart(MusECore::Track*)), MusEGlobal::muse, SLOT(startEditor(MusECore::Track*)));
@@ -112,7 +119,6 @@ ArrangerView::ArrangerView(QWidget* parent)
   connect(arranger, SIGNAL(toolChanged(int)), editTools, SLOT(set(int)));
   connect(MusEGlobal::muse, SIGNAL(configChanged()), arranger, SLOT(configChanged()));
   connect(arranger, SIGNAL(setUsedTool(int)), editTools, SLOT(set(int)));
-  connect(arranger, SIGNAL(selectionChanged()), SLOT(selectionChanged()));
   connect(MusEGlobal::song, SIGNAL(songChanged(MusECore::SongChangedFlags_t)), this, SLOT(songChanged(MusECore::SongChangedFlags_t)));
 
 
@@ -121,6 +127,7 @@ ArrangerView::ArrangerView(QWidget* parent)
 
 
   //-------- Edit Actions
+  editDeleteAction = new QAction(QIcon(*deleteIcon), tr("D&elete"), this);
   editCutAction = new QAction(QIcon(*editcutIconSet), tr("C&ut"), this);
   editCopyAction = new QAction(QIcon(*editcopyIconSet), tr("&Copy"), this);
   editCopyRangeAction = new QAction(QIcon(*editcopyIconSet), tr("Copy in range"), this);
@@ -194,6 +201,7 @@ ArrangerView::ArrangerView(QWidget* parent)
   menuEdit->addActions(MusEGlobal::undoRedo->actions());
   menuEdit->addSeparator();
 
+  menuEdit->addAction(editDeleteAction);
   menuEdit->addAction(editCutAction);
   menuEdit->addAction(editCopyAction);
   menuEdit->addAction(editCopyRangeAction);
@@ -279,6 +287,7 @@ ArrangerView::ArrangerView(QWidget* parent)
 
 
   //-------- Edit connections
+  connect(editDeleteAction, SIGNAL(triggered()), editSignalMapper, SLOT(map()));
   connect(editCutAction, SIGNAL(triggered()), editSignalMapper, SLOT(map()));
   connect(editCopyAction, SIGNAL(triggered()), editSignalMapper, SLOT(map()));
   connect(editCopyRangeAction, SIGNAL(triggered()), editSignalMapper, SLOT(map()));
@@ -302,6 +311,7 @@ ArrangerView::ArrangerView(QWidget* parent)
   connect(editOutsideLoopAction, SIGNAL(triggered()), editSignalMapper, SLOT(map()));
   connect(editAllPartsAction, SIGNAL(triggered()), editSignalMapper, SLOT(map()));
 
+  editSignalMapper->setMapping(editDeleteAction, CMD_DELETE);
   editSignalMapper->setMapping(editCutAction, CMD_CUT);
   editSignalMapper->setMapping(editCopyAction, CMD_COPY);
   editSignalMapper->setMapping(editCopyRangeAction, CMD_COPY_RANGE);
@@ -325,28 +335,28 @@ ArrangerView::ArrangerView(QWidget* parent)
 
   connect(editSignalMapper, SIGNAL(mapped(int)), this, SLOT(cmd(int)));
 
-  connect(startPianoEditAction, SIGNAL(activated()), MusEGlobal::muse, SLOT(startPianoroll()));
-  connect(startScoreEditAction, SIGNAL(activated()), MusEGlobal::muse, SLOT(startScoreQuickly()));
-  connect(startDrumEditAction, SIGNAL(activated()), MusEGlobal::muse, SLOT(startDrumEditor()));
-  connect(startListEditAction, SIGNAL(activated()), MusEGlobal::muse, SLOT(startListEditor()));
-  connect(startWaveEditAction, SIGNAL(activated()), MusEGlobal::muse, SLOT(startWaveEditor()));
+  connect(startPianoEditAction, SIGNAL(triggered()), MusEGlobal::muse, SLOT(startPianoroll()));
+  connect(startScoreEditAction, SIGNAL(triggered()), MusEGlobal::muse, SLOT(startScoreQuickly()));
+  connect(startDrumEditAction, SIGNAL(triggered()), MusEGlobal::muse, SLOT(startDrumEditor()));
+  connect(startListEditAction, SIGNAL(triggered()), MusEGlobal::muse, SLOT(startListEditor()));
+  connect(startWaveEditAction, SIGNAL(triggered()), MusEGlobal::muse, SLOT(startWaveEditor()));
   connect(scoreOneStaffPerTrackMapper, SIGNAL(mapped(QWidget*)), MusEGlobal::muse, SLOT(openInScoreEdit_oneStaffPerTrack(QWidget*)));
   connect(scoreAllInOneMapper, SIGNAL(mapped(QWidget*)), MusEGlobal::muse, SLOT(openInScoreEdit_allInOne(QWidget*)));
 
 
-  connect(masterGraphicAction, SIGNAL(activated()), MusEGlobal::muse, SLOT(startMasterEditor()));
-  connect(masterListAction, SIGNAL(activated()), MusEGlobal::muse, SLOT(startLMasterEditor()));
+  connect(masterGraphicAction, SIGNAL(triggered()), MusEGlobal::muse, SLOT(startMasterEditor()));
+  connect(masterListAction, SIGNAL(triggered()), MusEGlobal::muse, SLOT(startLMasterEditor()));
 
-  connect(midiTransformerAction, SIGNAL(activated()), MusEGlobal::muse, SLOT(startMidiTransformer()));
+  connect(midiTransformerAction, SIGNAL(triggered()), MusEGlobal::muse, SLOT(startMidiTransformer()));
 
 
   //-------- Structure connections
-  connect(strGlobalCutAction, SIGNAL(activated()), SLOT(globalCut()));
-  connect(strGlobalInsertAction, SIGNAL(activated()), SLOT(globalInsert()));
-  connect(strGlobalSplitAction, SIGNAL(activated()), SLOT(globalSplit()));
-  connect(strGlobalCutSelAction, SIGNAL(activated()), SLOT(globalCutSel()));
-  connect(strGlobalInsertSelAction, SIGNAL(activated()), SLOT(globalInsertSel()));
-  connect(strGlobalSplitSelAction, SIGNAL(activated()), SLOT(globalSplitSel()));
+  connect(strGlobalCutAction, SIGNAL(triggered()), SLOT(globalCut()));
+  connect(strGlobalInsertAction, SIGNAL(triggered()), SLOT(globalInsert()));
+  connect(strGlobalSplitAction, SIGNAL(triggered()), SLOT(globalSplit()));
+  connect(strGlobalCutSelAction, SIGNAL(triggered()), SLOT(globalCutSel()));
+  connect(strGlobalInsertSelAction, SIGNAL(triggered()), SLOT(globalInsertSel()));
+  connect(strGlobalSplitSelAction, SIGNAL(triggered()), SLOT(globalSplitSel()));
 
 
 
@@ -358,15 +368,6 @@ ArrangerView::ArrangerView(QWidget* parent)
   connect(cb, SIGNAL(selectionChanged()), SLOT(clipboardChanged()));
 
   finalizeInit();
-
-  // work around for probable QT/WM interaction bug.
-  // for certain window managers, e.g xfce, this window is
-  // is displayed although not specifically set to show();
-  // bug: 2811156     Softsynth GUI unclosable with XFCE4 (and a few others)
-  // Nov 21, 2012 Hey this causes the thing not to open at all, EVER, on Lubuntu and some others!
-  // And we had a request to remove this from a knowledgable tester. REMOVE Tim.
-  ///show();
-  ///hide();
 }
 
 ArrangerView::~ArrangerView()
@@ -396,6 +397,11 @@ void ArrangerView::songChanged(MusECore::SongChangedFlags_t type)
              SC_CONFIG | 
              SC_DRUMMAP)) 
     visTracks->updateVisibleTracksButtons();
+  
+  if(type & (SC_TRACK_SELECTION | SC_PART_SELECTION | 
+             SC_TRACK_INSERTED | SC_TRACK_REMOVED | SC_TRACK_MODIFIED | 
+             SC_PART_INSERTED | SC_PART_REMOVED | SC_PART_MODIFIED))
+    selectionChanged();
 }
       
 
@@ -486,6 +492,11 @@ void ArrangerView::writeConfiguration(int level, MusECore::Xml& xml)
 
 void ArrangerView::cmd(int cmd)
       {
+      // Don't process if user is dragging or has clicked on the parts. 
+      // Causes problems/crashes later in Canvas::viewMouseMoveEvent and viewMouseReleaseEvent.
+      if(arranger && arranger->getCanvas() && arranger->getCanvas()->getCurrentDrag())
+        return;
+      
       MusECore::TrackList* tracks = MusEGlobal::song->tracks();
       int l = MusEGlobal::song->lpos();
       int r = MusEGlobal::song->rpos();
@@ -519,26 +530,21 @@ void ArrangerView::cmd(int cmd)
                   arranger->cmd(Arranger::CMD_INSERT_EMPTYMEAS);
                   break;
             case CMD_DELETE:
-                  if (!MusEGlobal::song->msgRemoveParts()) //automatically does undo if neccessary and returns true then
+                  if (!MusECore::delete_selected_parts())
                   {
                       QMessageBox::StandardButton btn = QMessageBox::warning(
                           this,tr("Remove track(s)"),tr("Are you sure you want to remove this track(s)?"),
                           QMessageBox::Ok|QMessageBox::Cancel, QMessageBox::Ok);
 
                       if (btn == QMessageBox::Cancel)
-                          break;                            //msgRemoveParts() returned false -> no parts to remove?
-                      MusEGlobal::song->startUndo();
-                      MusEGlobal::audio->msgRemoveTracks(); //TODO FINDME this could still be speeded up!
-                      MusEGlobal::song->endUndo(SC_TRACK_REMOVED);
+                          break;
+                      MusEGlobal::audio->msgRemoveTracks();
                   }
 
                   break;
             case CMD_DELETE_TRACK: // from menu
                   {
-                  MusEGlobal::song->startUndo();
                   MusEGlobal::audio->msgRemoveTracks();
-                  MusEGlobal::song->endUndo(SC_TRACK_REMOVED);
-                  MusEGlobal::audio->msgUpdateSoloStates();
                   }
                   break;
 
@@ -628,13 +634,13 @@ void ArrangerView::updateScoreMenus()
 
   
   action=new QAction(tr("New"), this);
-  connect(action, SIGNAL(activated()), scoreOneStaffPerTrackMapper, SLOT(map()));
+  connect(action, SIGNAL(triggered()), scoreOneStaffPerTrackMapper, SLOT(map()));
   scoreOneStaffPerTrackMapper->setMapping(action, (QWidget*)NULL);
   scoreOneStaffPerTrackSubsubmenu->addAction(action);
   
   
   action=new QAction(tr("New"), this); //the above action may NOT be reused!
-  connect(action, SIGNAL(activated()), scoreAllInOneMapper, SLOT(map()));
+  connect(action, SIGNAL(triggered()), scoreAllInOneMapper, SLOT(map()));
   scoreAllInOneMapper->setMapping(action, (QWidget*)NULL);
   scoreAllInOneSubsubmenu->addAction(action);
 
@@ -646,13 +652,13 @@ void ArrangerView::updateScoreMenus()
       ScoreEdit* score = dynamic_cast<ScoreEdit*>(*it);
       
       action=new QAction(score->get_name(), this);
-      connect(action, SIGNAL(activated()), scoreOneStaffPerTrackMapper, SLOT(map()));
+      connect(action, SIGNAL(triggered()), scoreOneStaffPerTrackMapper, SLOT(map()));
       scoreOneStaffPerTrackMapper->setMapping(action, (QWidget*)score);
       scoreOneStaffPerTrackSubsubmenu->addAction(action);
 
 
       action=new QAction(score->get_name(), this); //the above action may NOT be reused!
-      connect(action, SIGNAL(activated()), scoreAllInOneMapper, SLOT(map()));
+      connect(action, SIGNAL(triggered()), scoreAllInOneMapper, SLOT(map()));
       scoreAllInOneMapper->setMapping(action, (QWidget*)score);
       scoreAllInOneSubsubmenu->addAction(action);
     }
@@ -675,14 +681,17 @@ void ArrangerView::populateAddTrack()
       QActionGroup *grp = MusEGui::populateAddTrack(addTrack, true, true);
       connect(addTrack, SIGNAL(triggered(QAction *)), SLOT(addNewTrack(QAction *)));
       
-      trackMidiAction = grp->actions()[0];
-      trackDrumAction = grp->actions()[1];
-      trackNewStyleDrumAction = grp->actions()[2];
-      trackWaveAction = grp->actions()[3];
-      trackAOutputAction = grp->actions()[4];
-      trackAGroupAction = grp->actions()[5];
-      trackAInputAction = grp->actions()[6];
-      trackAAuxAction = grp->actions()[7];
+      int idx = 0;
+      trackMidiAction = grp->actions()[idx++];
+      trackDrumAction = grp->actions()[idx++];
+      trackWaveAction = grp->actions()[idx++];
+      trackAOutputAction = grp->actions()[idx++];
+      trackAGroupAction = grp->actions()[idx++];
+      trackAInputAction = grp->actions()[idx++];
+      trackAAuxAction = grp->actions()[idx++];
+
+
+      arranger->getTrackList()->populateAddTrack();
 }
 
 void ArrangerView::addNewTrack(QAction* action)
@@ -692,6 +701,7 @@ void ArrangerView::addNewTrack(QAction* action)
 
 void ArrangerView::updateShortcuts()
 {
+      editDeleteAction->setShortcut(shortcuts[SHRT_DELETE].key);
       editCutAction->setShortcut(shortcuts[SHRT_CUT].key);
       editCopyAction->setShortcut(shortcuts[SHRT_COPY].key);
       editCopyRangeAction->setShortcut(shortcuts[SHRT_COPY_RANGE].key);
@@ -701,12 +711,12 @@ void ArrangerView::updateShortcuts()
       editPasteCloneToTrackAction->setShortcut(shortcuts[SHRT_PASTE_CLONE_TO_TRACK].key);
       editPasteDialogAction->setShortcut(shortcuts[SHRT_PASTE_DIALOG].key);
       editInsertEMAction->setShortcut(shortcuts[SHRT_INSERTMEAS].key);
+      editDuplicateSelTrackAction->setShortcut(shortcuts[SHRT_DUPLICATE_TRACK].key);
 
       //editDeleteSelectedAction has no acceleration
       
       trackMidiAction->setShortcut(shortcuts[SHRT_ADD_MIDI_TRACK].key);
       trackDrumAction->setShortcut(shortcuts[SHRT_ADD_DRUM_TRACK].key);
-      trackNewStyleDrumAction->setShortcut(shortcuts[SHRT_ADD_NEW_STYLE_DRUM_TRACK].key);
       trackWaveAction->setShortcut(shortcuts[SHRT_ADD_WAVE_TRACK].key);
       trackAOutputAction->setShortcut(shortcuts[SHRT_ADD_AUDIO_OUTPUT].key);
       trackAGroupAction->setShortcut(shortcuts[SHRT_ADD_AUDIO_GROUP].key);
@@ -755,14 +765,24 @@ void ArrangerView::clipboardChanged()
 
 //---------------------------------------------------------
 //   selectionChanged
+//   NOTE: This is received upon EITHER a part or track selection change from the Arranger.
 //---------------------------------------------------------
 
 void ArrangerView::selectionChanged()
       {
-      //bool flag = arranger->isSingleSelection();  // -- Hmm, why only single? 
-      bool flag = arranger->selectionSize() > 0;    // -- Test OK cut and copy. For muse2. Tim.
-      editCutAction->setEnabled(flag);
-      editCopyAction->setEnabled(flag);
+      bool pflag = arranger->itemsAreSelected();
+      bool tflag = MusECore::tracks_are_selected();
+
+      editDeleteAction->setEnabled(tflag || pflag);
+
+      editDeleteSelectedAction->setEnabled(tflag);
+      editDuplicateSelTrackAction->setEnabled(tflag);
+   
+      editCutAction->setEnabled(pflag);
+      editCopyAction->setEnabled(pflag);
+      editShrinkPartsAction->setEnabled(pflag);
+      editExpandPartsAction->setEnabled(pflag);
+      editCleanPartsAction->setEnabled(pflag);
       }
 
 
