@@ -102,7 +102,7 @@ void MidiControllerEditDialog::addControllerToView(MidiController* mctrl)
 //   MidiControllerEditDialog
 //---------------------------------------------------------
 
-MidiControllerEditDialog::MidiControllerEditDialog(QWidget* parent, const char* name, bool modal, Qt::WFlags fl)
+MidiControllerEditDialog::MidiControllerEditDialog(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
    : MidiControllerEditDialogBase(parent, name, modal, fl)
       {
       _lastPort = midiPortsList->currentItem();
@@ -178,9 +178,7 @@ void MidiControllerEditDialog::updateMidiPortsList()
       for (int i = 0; i < MIDI_PORTS; ++i) {
             MidiPort* port = &midiPorts[i];
             MidiDevice* dev = port->device();
-            QString name;
-            name.sprintf("%d(%s)", port->portno()+1,
-                     dev ? dev->name().toLatin1() : "none");
+            QString name = QString("%1(%2)").arg(port->portno() + 1).arg(dev ? dev->name() : QString("none"));
             midiPortsList->insertItem(name, i);
             }
       _lastPort = midiPortsList->currentItem();
