@@ -116,7 +116,18 @@ MasterEdit::MasterEdit(QWidget* parent, const char* name)
       settingsMenu->addAction(fullscreenAction);
 
       // Toolbars ---------------------------------------------------------
-      MusEGui::EditToolBar* tools2 = new MusEGui::EditToolBar(this, MusEGui::PointerTool | MusEGui::PencilTool | MusEGui::RubberTool);
+
+      // NOTICE: Please ensure that any tool bar object names here match the names assigned 
+      //          to identical or similar toolbars in class MusE or other TopWin classes. 
+      //         This allows MusE::setCurrentMenuSharingTopwin() to do some magic
+      //          to retain the original toolbar layout. If it finds an existing
+      //          toolbar with the same object name, it /replaces/ it using insertToolBar(),
+      //          instead of /appending/ with addToolBar().
+
+      addToolBarBreak();
+      
+      // Already has an object name.
+      MusEGui::EditToolBar* tools2 = new MusEGui::EditToolBar(this, MusEGui::PointerTool | MusEGui::PencilTool | MusEGui::RubberTool| MusEGui::DrawTool);
       addToolBar(tools2);
 
       QToolBar* enableMaster = addToolBar(tr("Enable master"));
@@ -131,7 +142,8 @@ MasterEdit::MasterEdit(QWidget* parent, const char* name)
       connect(enableButton, SIGNAL(toggled(bool)), MusEGlobal::song, SLOT(setMasterFlag(bool)));
 
       QToolBar* info = addToolBar(tr("Info"));
-      info->setObjectName("Info");
+      // Make it appear like a Toolbar1 object.
+      info->setObjectName("Pos/Snap/Solo-tools");
       QLabel* label  = new QLabel(tr("Cursor"));
       label->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
       label->setIndent(3);
@@ -285,13 +297,14 @@ void MasterEdit::readStatus(MusECore::Xml& xml)
                   case MusECore::Xml::TagStart:
                         if (tag == "midieditor")
                               MidiEditor::readStatus(xml);
+                        else if (tag == "xpos")
+                              hscroll->setPos(xml.parseInt());
+                        else if (tag == "xmag") 
+                              hscroll->setMag(xml.parseInt());
                         else if (tag == "ypos")
                               vscroll->setPos(xml.parseInt());
-                        else if (tag == "ymag") {
-                              // vscroll->setMag(xml.parseInt());
-                              int mag = xml.parseInt();
-                              vscroll->setMag(mag);
-                              }
+                        else if (tag == "ymag") 
+                              vscroll->setMag(xml.parseInt());
                         else
                               xml.unknown("MasterEdit");
                         break;
@@ -324,8 +337,10 @@ void MasterEdit::readStatus(MusECore::Xml& xml)
 void MasterEdit::writeStatus(int level, MusECore::Xml& xml) const
       {
       xml.tag(level++, "master");
-      xml.intTag(level, "ypos", vscroll->pos());
+      xml.intTag(level, "xmag", hscroll->mag());
+      xml.intTag(level, "xpos", hscroll->pos());
       xml.intTag(level, "ymag", vscroll->mag());
+      xml.intTag(level, "ypos", vscroll->pos());
       MidiEditor::writeStatus(level, xml);
       xml.tag(level, "/master");
       }
