@@ -47,9 +47,14 @@ static int rtcResolutions[] = {
 static int divisions[] = {
       48, 96, 192, 384, 768, 1536, 3072, 6144, 12288
       };
-static int dummyAudioBufSizes[] = {
+static int selectableAudioBufSizes[] = {
       16, 32, 64, 128, 256, 512, 1024, 2048
       };
+static int selectableAudioSampleRates[] = {
+      22050, 32000, 44100, 48000, 64000, 88200, 96000, 192000
+      };
+int numAudioSampleRates = 8;
+
 static unsigned long minControlProcessPeriods[] = {
       1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048
       };
@@ -60,7 +65,7 @@ static unsigned long minControlProcessPeriods[] = {
 
 GlobalSettingsConfig::GlobalSettingsConfig(QWidget* parent)
    : QDialog(parent)
-      {
+{
       setupUi(this);
       startSongGroup = new QButtonGroup(this);
       startSongGroup->addButton(startLastButton, 0);
@@ -94,6 +99,12 @@ GlobalSettingsConfig::GlobalSettingsConfig(QWidget* parent)
       connect(buttonTraditionalPreset, SIGNAL(clicked()), SLOT(traditionalPreset()));
       connect(buttonMDIPreset, SIGNAL(clicked()), SLOT(mdiPreset()));
       connect(buttonBorlandPreset, SIGNAL(clicked()), SLOT(borlandPreset()));
+
+      connect(pluginPathAdd, SIGNAL(clicked()), SLOT(addPluginPath()));
+      connect(pluginPathEdit, SIGNAL(clicked()), SLOT(editPluginPath()));
+      connect(pluginPathRemove, SIGNAL(clicked()), SLOT(removePluginPath()));
+      connect(pluginPathMoveUp, SIGNAL(clicked()), SLOT(movePluginPathUp()));
+      connect(pluginPathMoveDown, SIGNAL(clicked()), SLOT(movePluginPathDown()));
       
       addMdiSettings(TopWin::ARRANGER);
       addMdiSettings(TopWin::SCORE);
@@ -105,8 +116,18 @@ GlobalSettingsConfig::GlobalSettingsConfig(QWidget* parent)
       addMdiSettings(TopWin::LMASTER);
       addMdiSettings(TopWin::CLIPLIST);
       addMdiSettings(TopWin::MARKER);
-      
+
+      for (int i = 0; i < MusEGlobal::numRtAudioDevices; i++){
+        deviceAudioBackendComboBox->addItem(MusEGlobal::selectableAudioBackendDevices[i],i);
       }
+#ifndef HAVE_RTAUDIO
+      deviceAudioBackendComboBox->setDisabled(true);
+#endif
+
+      for (int i = 0; i < numAudioSampleRates; i++){
+        deviceAudioRate->addItem(QString::number(selectableAudioSampleRates[i]),i);
+      }
+}
 
 void GlobalSettingsConfig::addMdiSettings(TopWin::ToplevelType t)
 {
@@ -139,13 +160,19 @@ void GlobalSettingsConfig::updateSettings()
                   break;
                   }
             }
-      for (unsigned i = 0; i < sizeof(dummyAudioBufSizes)/sizeof(*dummyAudioBufSizes); ++i) {
-            if (dummyAudioBufSizes[i] == MusEGlobal::config.dummyAudioBufSize) {
-                  dummyAudioSize->setCurrentIndex(i);
+      for (unsigned i = 0; i < sizeof(selectableAudioBufSizes)/sizeof(*selectableAudioBufSizes); ++i) {
+            if (selectableAudioBufSizes[i] == MusEGlobal::config.deviceAudioBufSize) {
+                  deviceAudioSize->setCurrentIndex(i);
                   break;
                   }
             }
       
+      for (int i = 0; i < numAudioSampleRates; ++i) {
+            if (selectableAudioSampleRates[i] == MusEGlobal::config.deviceAudioSampleRate) {
+                  deviceAudioRate->setCurrentIndex(i);
+                  break;
+                  }
+            }
       for (unsigned i = 0; i < sizeof(minControlProcessPeriods)/sizeof(*minControlProcessPeriods); ++i) {
             if (minControlProcessPeriods[i] == MusEGlobal::config.minControlProcessPeriod) {
                   minControlProcessPeriodComboBox->setCurrentIndex(i);
@@ -153,10 +180,22 @@ void GlobalSettingsConfig::updateSettings()
                   }
             }
 
-      warnIfBadTimingCheckBox->setChecked(MusEGlobal::config.warnIfBadTiming);      
+      autoSaveCheckBox->setChecked(MusEGlobal::config.autoSave);
+      scrollableSubmenusCheckbox->setChecked(MusEGlobal::config.scrollableSubMenus);
+      liveWaveUpdateCheckBox->setChecked(MusEGlobal::config.liveWaveUpdate);
+      preferKnobsVsSlidersCheckBox->setChecked(MusEGlobal::config.preferKnobsVsSliders);
+      showControlValuesCheckBox->setChecked(MusEGlobal::config.showControlValues);
+      monitorOnRecordCheckBox->setChecked(MusEGlobal::config.monitorOnRecord);
+      lineEditStyleHackCheckBox->setChecked(MusEGlobal::config.lineEditStyleHack);
+      showNoteNamesCheckBox->setChecked(MusEGlobal::config.showNoteNamesInPianoRoll);
+      preferMidiVolumeDbCheckBox->setChecked(MusEGlobal::config.preferMidiVolumeDb);
+      warnIfBadTimingCheckBox->setChecked(MusEGlobal::config.warnIfBadTiming);
+      warnOnFileVersionsCheckBox->setChecked(MusEGlobal::config.warnOnFileVersions);
       midiSendInit->setChecked(MusEGlobal::config.midiSendInit);      
       midiWarnInitPending->setChecked(MusEGlobal::config.warnInitPending);      
       midiSendCtlDefaults->setChecked(MusEGlobal::config.midiSendCtlDefaults);      
+      sendNullParamsCB->setChecked(MusEGlobal::config.midiSendNullParameters);      
+      optimizeControllersCB->setChecked(MusEGlobal::config.midiOptimizeControllers);      
       guiRefreshSelect->setValue(MusEGlobal::config.guiRefresh);
       minSliderSelect->setValue(int(MusEGlobal::config.minSlider));
       minMeterSelect->setValue(MusEGlobal::config.minMeter);
@@ -164,15 +203,12 @@ void GlobalSettingsConfig::updateSettings()
       denormalCheckBox->setChecked(MusEGlobal::config.useDenormalBias);
       outputLimiterCheckBox->setChecked(MusEGlobal::config.useOutputLimiter);
       vstInPlaceCheckBox->setChecked(MusEGlobal::config.vstInPlace);
-      dummyAudioRate->setValue(MusEGlobal::config.dummyAudioSampleRate);
-      
-      //DummyAudioDevice* dad = dynamic_cast<DummyAudioDevice*>(audioDevice);
-      //dummyAudioRealRate->setText(dad ? QString().setNum(sampleRate) : "---");
-      //dummyAudioRealRate->setText(QString().setNum(sampleRate));   // Not used any more. p4.0.20  DELETETHIS?
-      
+
+      deviceAudioBackendComboBox->setCurrentIndex(MusEGlobal::config.deviceAudioBackend);
+
       projDirEntry->setText(MusEGlobal::config.projectBaseFolder);
 
-      startSongEntry->setText(MusEGlobal::config.startSong);
+      startSongEntry->setText(MusEGlobal::config.startSong == "" ? "<default>" : MusEGlobal::config.startSong);
       startSongGroup->button(MusEGlobal::config.startMode)->setChecked(true);
       readMidiConfigFromSongCheckBox->setChecked(MusEGlobal::config.startSongLoadConfig);
       
@@ -211,6 +247,7 @@ void GlobalSettingsConfig::updateSettings()
       setBigtimeCurrent->setEnabled(MusEGlobal::muse->bigtimeWindow());
       setTransportCurrent->setEnabled(MusEGlobal::muse->transportWindow());
 
+      fixFrozenMDISubWindowsCheckBox->setChecked(MusEGlobal::config.fixFrozenMDISubWindows);
       showSplash->setChecked(MusEGlobal::config.showSplashScreen);
       showDidYouKnow->setChecked(MusEGlobal::config.showDidYouKnow);
       externalWavEditorSelect->setText(MusEGlobal::config.externalWavEditor);
@@ -237,6 +274,23 @@ void GlobalSettingsConfig::updateSettings()
 
       trackHeight->setValue(MusEGlobal::config.trackHeight);
 
+      lv2UiBehaviorComboBox->setCurrentIndex(static_cast<int>(MusEGlobal::config.lv2UiBehavior));
+
+      pluginLadspaPathList->clear();
+      pluginLadspaPathList->addItems(MusEGlobal::config.pluginLadspaPathList);
+
+      pluginDssiPathList->clear();
+      pluginDssiPathList->addItems(MusEGlobal::config.pluginDssiPathList);
+
+      pluginVstPathList->clear();
+      pluginVstPathList->addItems(MusEGlobal::config.pluginVstPathList);
+
+      pluginLinuxVstPathList->clear();
+      pluginLinuxVstPathList->addItems(MusEGlobal::config.pluginLinuxVstPathList);
+
+      pluginLv2PathList->clear();
+      pluginLv2PathList->addItems(MusEGlobal::config.pluginLv2PathList);
+
       updateMdiSettings();
 }
 
@@ -278,22 +332,28 @@ void GlobalSettingsConfig::apply()
       MusEGlobal::config.vstInPlace  = vstInPlaceCheckBox->isChecked();
       MusEGlobal::config.rtcTicks    = rtcResolutions[rtcticks];
       MusEGlobal::config.warnIfBadTiming = warnIfBadTimingCheckBox->isChecked();
+      MusEGlobal::config.warnOnFileVersions = warnOnFileVersionsCheckBox->isChecked();
       MusEGlobal::config.midiSendInit = midiSendInit->isChecked();
       MusEGlobal::config.warnInitPending = midiWarnInitPending->isChecked();
       MusEGlobal::config.midiSendCtlDefaults = midiSendCtlDefaults->isChecked();
+      MusEGlobal::config.midiSendNullParameters = sendNullParamsCB->isChecked();
+      MusEGlobal::config.midiOptimizeControllers = optimizeControllersCB->isChecked();
       
       MusEGlobal::config.projectBaseFolder = projDirEntry->text();
       
-      MusEGlobal::config.startSong   = startSongEntry->text();
+      MusEGlobal::config.startSong   = startSongEntry->text() == "<default>" ? "" : startSongEntry->text();
       MusEGlobal::config.startMode   = startSongGroup->checkedId();
       MusEGlobal::config.startSongLoadConfig = readMidiConfigFromSongCheckBox->isChecked();
 
       MusEGlobal::config.newDrumRecordCondition = MusECore::newDrumRecordCondition_t(recDrumGroup->checkedId());
 
       
-      int das = dummyAudioSize->currentIndex();
-      MusEGlobal::config.dummyAudioBufSize = dummyAudioBufSizes[das];
-      MusEGlobal::config.dummyAudioSampleRate = dummyAudioRate->value();
+      int das = deviceAudioSize->currentIndex();
+      MusEGlobal::config.deviceAudioBufSize = selectableAudioBufSizes[das];
+      MusEGlobal::config.deviceAudioSampleRate = selectableAudioSampleRates[deviceAudioRate->currentIndex()];
+
+      MusEGlobal::config.deviceAudioBackend = deviceAudioBackendComboBox->currentIndex();
+
       int mcp = minControlProcessPeriodComboBox->currentIndex();
       MusEGlobal::config.minControlProcessPeriod = minControlProcessPeriods[mcp];
 
@@ -331,6 +391,16 @@ void GlobalSettingsConfig::apply()
       MusEGlobal::config.mixer2.geometry.setWidth(mixer2W->value());
       MusEGlobal::config.mixer2.geometry.setHeight(mixer2H->value());
 
+      MusEGlobal::config.fixFrozenMDISubWindows = fixFrozenMDISubWindowsCheckBox->isChecked();
+      MusEGlobal::config.autoSave = autoSaveCheckBox->isChecked();
+      MusEGlobal::config.scrollableSubMenus = scrollableSubmenusCheckbox->isChecked();
+      MusEGlobal::config.liveWaveUpdate = liveWaveUpdateCheckBox->isChecked();
+      MusEGlobal::config.preferKnobsVsSliders = preferKnobsVsSlidersCheckBox->isChecked();
+      MusEGlobal::config.showControlValues = showControlValuesCheckBox->isChecked();
+      MusEGlobal::config.monitorOnRecord = monitorOnRecordCheckBox->isChecked();
+      MusEGlobal::config.lineEditStyleHack = lineEditStyleHackCheckBox->isChecked();
+      MusEGlobal::config.showNoteNamesInPianoRoll = showNoteNamesCheckBox->isChecked();
+      MusEGlobal::config.preferMidiVolumeDb = preferMidiVolumeDbCheckBox->isChecked();
       MusEGlobal::config.showSplashScreen = showSplash->isChecked();
       MusEGlobal::config.showDidYouKnow   = showDidYouKnow->isChecked();
       MusEGlobal::config.externalWavEditor = externalWavEditorSelect->text();
@@ -372,11 +442,15 @@ void GlobalSettingsConfig::apply()
             w->resize(MusEGlobal::config.geometryBigTime.size());
             w->move(MusEGlobal::config.geometryBigTime.topLeft());
             }
-      MusEGlobal::muse->resize(MusEGlobal::config.geometryMain.size());
-      MusEGlobal::muse->move(MusEGlobal::config.geometryMain.topLeft());
+      if(!MusEGlobal::muse->isMaximized() && !MusEGlobal::muse->isMinimized())
+      {
+         MusEGlobal::muse->resize(MusEGlobal::config.geometryMain.size());
+         MusEGlobal::muse->move(MusEGlobal::config.geometryMain.topLeft());
+      }
 
       MusEGlobal::muse->setHeartBeat();        // set guiRefresh
-      MusEGlobal::midiSeq->msgSetRtc();        // set midi tick rate
+      if(MusEGlobal::midiSeq)
+        MusEGlobal::midiSeq->msgSetRtc();        // set midi tick rate
       
       if (onlyNewDrumBtn->isChecked())
         MusEGlobal::config.drumTrackPreference=MusEGlobal::ONLY_NEW;
@@ -389,9 +463,34 @@ void GlobalSettingsConfig::apply()
 
       MusEGlobal::config.trackHeight = trackHeight->value();
 
+      MusEGlobal::config.lv2UiBehavior = static_cast<MusEGlobal::CONF_LV2_UI_BEHAVIOR>(lv2UiBehaviorComboBox->currentIndex());
+
+      
+      MusEGlobal::config.pluginLadspaPathList.clear();
+      for (int i = 0; i < pluginLadspaPathList->count(); ++i)
+            MusEGlobal::config.pluginLadspaPathList << pluginLadspaPathList->item(i)->text();
+
+      MusEGlobal::config.pluginDssiPathList.clear();
+      for (int i = 0; i < pluginDssiPathList->count(); ++i)
+            MusEGlobal::config.pluginDssiPathList << pluginDssiPathList->item(i)->text();
+
+      MusEGlobal::config.pluginVstPathList.clear();
+      for (int i = 0; i < pluginVstPathList->count(); ++i)
+            MusEGlobal::config.pluginVstPathList << pluginVstPathList->item(i)->text();
+
+      MusEGlobal::config.pluginLinuxVstPathList.clear();
+      for (int i = 0; i < pluginLinuxVstPathList->count(); ++i)
+            MusEGlobal::config.pluginLinuxVstPathList << pluginLinuxVstPathList->item(i)->text();
+
+      MusEGlobal::config.pluginLv2PathList.clear();
+      for (int i = 0; i < pluginLv2PathList->count(); ++i)
+            MusEGlobal::config.pluginLv2PathList << pluginLv2PathList->item(i)->text();
+      
       applyMdiSettings();
       
-      MusEGlobal::muse->changeConfig(true);    // save settings
+      // Save settings. Use simple version - do NOT set style or stylesheet, this has nothing to do with that.
+      MusEGlobal::muse->changeConfig(true);
+      raise();
       }
 
 //---------------------------------------------------------
@@ -525,6 +624,267 @@ void GlobalSettingsConfig::borlandPreset()
   updateMdiSettings();
 }
 
+void GlobalSettingsConfig::addPluginPath()
+{
+  QString path; 
+  switch(pluginPathsTabs->currentIndex())
+  {
+    case LadspaTab:
+      if(pluginLadspaPathList->currentItem())
+        path = pluginLadspaPathList->currentItem()->text();
+    break;
+    
+    case DssiTab:
+      if(pluginDssiPathList->currentItem())
+        path = pluginDssiPathList->currentItem()->text();
+    break;
+    
+    case VstTab:
+      if(pluginVstPathList->currentItem())
+        path = pluginVstPathList->currentItem()->text();
+    break;
+    
+    case LinuxVstTab:
+      if(pluginLinuxVstPathList->currentItem())
+        path = pluginLinuxVstPathList->currentItem()->text();
+    break;
+    
+    case Lv2Tab:
+      if(pluginLv2PathList->currentItem())
+        path = pluginLv2PathList->currentItem()->text();
+    break;
+    
+    default:
+    break;
+  }
+  
+  QString new_path = browsePluginPath(path);
+  
+  if(new_path.isEmpty())
+    return;
+  
+  switch(pluginPathsTabs->currentIndex())
+  {
+    case LadspaTab:
+      pluginLadspaPathList->addItem(new_path);
+    break;
+    
+    case DssiTab:
+      pluginDssiPathList->addItem(new_path);
+    break;
+    
+    case VstTab:
+      pluginVstPathList->addItem(new_path);
+    break;
+    
+    case LinuxVstTab:
+      pluginLinuxVstPathList->addItem(new_path);
+    break;
+    
+    case Lv2Tab:
+      pluginLv2PathList->addItem(new_path);
+    break;
+    
+    default:
+    break;
+  }
+}
+
+void GlobalSettingsConfig::editPluginPath()
+{
+  QString path; 
+  switch(pluginPathsTabs->currentIndex())
+  {
+    case LadspaTab:
+      if(pluginLadspaPathList->currentItem())
+        path = pluginLadspaPathList->currentItem()->text();
+    break;
+    
+    case DssiTab:
+      if(pluginDssiPathList->currentItem())
+        path = pluginDssiPathList->currentItem()->text();
+    break;
+    
+    case VstTab:
+      if(pluginVstPathList->currentItem())
+        path = pluginVstPathList->currentItem()->text();
+    break;
+    
+    case LinuxVstTab:
+      if(pluginLinuxVstPathList->currentItem())
+        path = pluginLinuxVstPathList->currentItem()->text();
+    break;
+    
+    case Lv2Tab:
+      if(pluginLv2PathList->currentItem())
+        path = pluginLv2PathList->currentItem()->text();
+    break;
+    
+    default:
+    break;
+  }
+  
+  QString new_path = browsePluginPath(path);
+  
+  if(new_path.isEmpty())
+    return;
+  
+  switch(pluginPathsTabs->currentIndex())
+  {
+    case LadspaTab:
+      if(pluginLadspaPathList->currentItem())
+        pluginLadspaPathList->currentItem()->setText(new_path);
+    break;
+    
+    case DssiTab:
+      if(pluginDssiPathList->currentItem())
+        pluginDssiPathList->currentItem()->setText(new_path);
+    break;
+    
+    case VstTab:
+      if(pluginVstPathList->currentItem())
+        pluginVstPathList->currentItem()->setText(new_path);
+    break;
+    
+    case LinuxVstTab:
+      if(pluginLinuxVstPathList->currentItem())
+        pluginLinuxVstPathList->currentItem()->setText(new_path);
+    break;
+    
+    case Lv2Tab:
+      if(pluginLv2PathList->currentItem())
+        pluginLv2PathList->currentItem()->setText(new_path);
+    break;
+    
+    default:
+    break;
+  }
+}
+
+QString GlobalSettingsConfig::browsePluginPath(const QString& path)
+{
+  QString dir = QFileDialog::getExistingDirectory(this, 
+                                                  qApp->translate("@default", 
+                                                    QT_TRANSLATE_NOOP("@default", 
+                                                                      "Select plugin directory")), 
+                                                  path);
+  return dir;
+}
+
+void GlobalSettingsConfig::removePluginPath()
+{
+  switch(pluginPathsTabs->currentIndex())
+  {
+    case LadspaTab:
+      foreach(QListWidgetItem* item, pluginLadspaPathList->selectedItems())
+        delete item;
+    break;
+    
+    case DssiTab:
+      foreach(QListWidgetItem* item, pluginDssiPathList->selectedItems())
+        delete item;
+    break;
+    
+    case VstTab:
+      foreach(QListWidgetItem* item, pluginVstPathList->selectedItems())
+        delete item;
+    break;
+    
+    case LinuxVstTab:
+      foreach(QListWidgetItem* item, pluginLinuxVstPathList->selectedItems())
+        delete item;
+    break;
+    
+    case Lv2Tab:
+      foreach(QListWidgetItem* item, pluginLv2PathList->selectedItems())
+        delete item;
+    break;
+    
+    default:
+    return;
+  }
+}
+
+void GlobalSettingsConfig::movePluginPathUp()
+{
+  QListWidget* list = 0;
+  switch(pluginPathsTabs->currentIndex())
+  {
+    case LadspaTab:
+      list = pluginLadspaPathList;
+    break;
+    
+    case DssiTab:
+      list = pluginDssiPathList;
+    break;
+    
+    case VstTab:
+      list = pluginVstPathList;
+    break;
+    
+    case LinuxVstTab:
+      list = pluginLinuxVstPathList;
+    break;
+    
+    case Lv2Tab:
+      list = pluginLv2PathList;
+    break;
+    
+    default:
+    break;
+  }
+
+  if(list)
+  {
+    int row = list->currentRow();
+    if(row > 0)
+    {
+      list->insertItem(row - 1, list->takeItem(row));
+      list->setCurrentRow(row - 1);
+    }
+  }
+}
+
+void GlobalSettingsConfig::movePluginPathDown()
+{
+  QListWidget* list = 0;
+  switch(pluginPathsTabs->currentIndex())
+  {
+    case LadspaTab:
+      list = pluginLadspaPathList;
+    break;
+    
+    case DssiTab:
+      list = pluginDssiPathList;
+    break;
+    
+    case VstTab:
+      list = pluginVstPathList;
+    break;
+    
+    case LinuxVstTab:
+      list = pluginLinuxVstPathList;
+    break;
+    
+    case Lv2Tab:
+      list = pluginLv2PathList;
+    break;
+    
+    default:
+    break;
+  }
+
+  if(list)
+  {
+    int row = list->currentRow();
+    if(row + 1 < list->count())
+    {
+      list->insertItem(row + 1, list->takeItem(row));
+      list->setCurrentRow(row + 1);
+    }
+  }
+}
+
 void GlobalSettingsConfig::browseProjDir()
 {
   QString dir = MusEGui::browseProjectFolder(this);
@@ -547,7 +907,7 @@ void GlobalSettingsConfig::browseStartSongFile()
 
 void GlobalSettingsConfig::startSongReset()
 {
-  startSongEntry->setText(MusEGlobal::museGlobalShare + QString("/templates/default.med"));
+  startSongEntry->setText("<default>");
   readMidiConfigFromSongCheckBox->setChecked(false);
 }
 
